#!/usr/bin/env python
"""File for automising the translation process.
Run this when you need updated .po and .pot files.
Run this when you need updated .mo files.
Add languages by modifying the languages variable.
Automatically adds new files to bzr, unless --ignore-bzr is specified.
NOTE: you will need to install intltool and po4a for this to work.
"""

import os, sys
import subprocess
from xml.dom import minidom

# All supported languages should be added here and in ../po4a.cfg
languages = ("en_GB",) #("fr", "en_GB", "da", "es")

def main():
    if '--help' in sys.argv[1:]:
        usage()
        sys.exit(0)
    print "languages: %s" % " ".join(languages)
    gpot, gpo, gmo, addBzr = (False, False, False, True)
    if '--generate-pot' in sys.argv[1:]:
        gpot = True
    if '--generate-po' in sys.argv[1:]:
        gpo = True
    if '--generate-mo' in sys.argv[1:]: # this also includes docbook documentation
        gmo = True
    if not True in (gpot, gpo, gmo):
        print "Defaulting to --generate-pot --generate-po"
        gpot, gpo, gmo = (True, True, False)
    if "--ignore-bzr" in sys.argv[1:]:
        addBzr = False
    
    dp = determine_path()
    pp = os.path.join(dp[:-1*len(dp.split("/")[-1])], "epidermis")
    rp = dp[:-1*len(dp.split("/")[-1])]
    # pp should be the epidermis dir with all the .py files in it
    # rp is the root directory of the branch
    
    poDir = os.path.join(dp[:-1*len(dp.split("/")[-1])], "po")
    
    if gpot:
        # make pot file
        
        call("mkdir -p %s" % poDir, shell=True)
        
        call("xgettext --language=Python --keyword=_ --from-code=UTF-8 --output=%s/epidermis.pot *.py pigments/*.py" %
            poDir, shell=True, cwd=pp)
        
        # add glade files
        call("xgettext --language=Glade --from-code=UTF-8 --output=glade.pot *.glade", shell=True, cwd=os.path.join(pp,"data"))
        call("msgcat --use-first po/epidermis.pot epidermis/data/glade.pot -o po/epidermis.pot",shell=True,cwd=rp)
            
        # add documentation to pot file
        call("po4a-gettextize -f docbook -m help/C/epidermis-doc.xml -p help/epidermis-doc.pot", shell=True, cwd=rp)
        call("msgcat --use-first po/epidermis.pot help/epidermis-doc.pot -o po/epidermis.pot", shell=True, cwd=rp)
        call("po4a-gettextize -f man -m linux/epidermis.1 -o groff_code=verbatim -p linux/epidermis-man.pot", shell=True, cwd=rp)
        call("msgcat --use-first po/epidermis.pot linux/epidermis-man.pot -o po/epidermis.pot", shell=True, cwd=rp)
        call("po4a-gettextize -f xml -o \"translated=<title> <description>\" -o \"untranslated=<creator> <date> <type>\" -o \"attributes=<language>code <identifier>url\" -m help/C/epidermis-C.omf -p help/epidermis-omf.pot", shell=True, cwd=rp)
        call("msgcat --use-first po/epidermis.pot help/epidermis-omf.pot -o po/epidermis.pot", shell=True, cwd=rp)
        call("rm help/epidermis-doc.pot linux/epidermis-man.pot help/epidermis-omf.pot epidermis/data/glade.pot", shell=True, cwd=rp)
        
        # bzr add the final pot file
        if addBzr: call("bzr add %s/epidermis.pot" % poDir, shell=True, cwd=pp)
    
    for lang in languages:
        if gpo and not os.path.exists(os.path.join(poDir, lang + ".po")):
            # note: cwd is poDir here
            call("msginit --input=epidermis.pot --locale=%s --no-translator" % lang, shell=True, cwd=poDir)
            if addBzr: call("bzr add %s/%s.po" % (poDir, lang), shell=True, cwd=pp)
        elif gpo:
            # note: cwd is poDir here
            call("msgmerge -U %s.po epidermis.pot" % lang, shell=True, cwd=poDir)
        # make .mo files
        if gmo:
            call("mkdir -p %s/%s/LC_MESSAGES" % (poDir, lang), shell=True, cwd=pp)
            call("msgfmt -o %s/%s/LC_MESSAGES/epidermis.mo %s/%s.po" % (poDir, lang, poDir, lang), shell=True, cwd=pp)
            print "%s/LC_MESSAGES/epidermis.mo done" % lang
            call("cp po/%s.po help/%s.po" % (lang,lang), shell=True, cwd=rp)
    if gmo:
        # generate docbooks and man pages
        call("po4a po4a.cfg", shell=True, cwd=rp)
        for lang in languages:
            # copy documentation screenshots over
            if os.path.exists(os.path.join(rp, "po/help/%s/images" % lang)):
                call("cp -rT po/help/%s/images help/%s/images" % (lang, lang), shell=True, cwd=rp)
            else:
                print "WARNING: copying english screenshots to %s translation" % lang
                call("cp -rT help/C/images help/%s/images" % lang, shell=True, cwd=rp)
            call("rm help/%s.po" % lang, shell=True, cwd=rp)
            # modify epidermis-lang.omf so that it points to the correct documentatian
            omf = minidom.parse(os.path.join(rp, "help/%s/epidermis-%s.omf" % (lang,lang)))
            omf.getElementsByTagName("identifier")[0].setAttribute("url","file:///usr/share/gnome/help/epidermis/%s/epidermis-doc.xml" % lang)
            omf.getElementsByTagName("language")[0].setAttribute("code", lang)
            omf.writexml(open(os.path.join(rp, "help/%s/epidermis-%s.omf" % (lang,lang)), "w"))
            del(omf)
        
        print "generated docbooks and man pages"

def call(*args, **kwargs):
    """a wrapper round subprocess.call, raises an Exception when the command returns
    a non-null error value"""
    rv = subprocess.call(*args, **kwargs)
    if rv != 0:
        class BadCall(Exception):
            def __init__(self, args, kwargs):
                self.args, self.kwargs = args, kwargs
            def __str__(self):
                return "BadCall; args: %s kwargs: %s" % (str(args), str(kwargs))
        raise(BadCall(args,kwargs))

def usage():
    """print help text about the script"""
    print """translate.py
--generate-pot: create gettext portable object template file epidermis.pot
--generate-po: create gettext portable objects (.po) for all supported languages
--generate-mo: create gettext binary portable objects (.mo) for all supported languages
If none of the above options are specified, the first two will be used. However, if one
or more of the options are specified, then the others won't be.
--help: display this message
--ignore-bzr: do not add new generated files to the bazaar branch"""
        
    
def determine_path ():
    """Borrowed from wxglade.py"""
    try:
        root = __file__
        if os.path.islink (root):
            root = os.path.realpath (root)
        return os.path.dirname (os.path.abspath (root))
    except Exception, ee:
        print >> sys.stderr, "I'm sorry, but something is wrong."
        print >> sys.stderr, "There is no __file__ variable. Please contact the author."
        sys.exit (1)
        raise(ee)

if __name__ == "__main__":
    main()
